@@ -1,4 +1,4 @@
-/*  $Id: innxbatch.c,v 1.10.2.1 2000/06/30 15:15:15 kondou Exp $
+/*  $Id: innxbatch.c 10153 2017-06-05 12:28:01Z iulius $
 **
 **  Transmit batches to remote site, using the XBATCH command
 **  Modelled after innxmit.c and nntpbatch.c
@@ -27,76 +27,69 @@
 **  To prevent this, innxbatch should be invoked by a shell script that uses
 **  shlock(1) to achieve mutual exclusion.
 */
+
 #include "config.h"
 #include "clibrary.h"
+#include "portable/socket.h"
 #include <ctype.h>
 #include <errno.h>
+#include <fcntl.h>
 #include <setjmp.h>
 #include <signal.h>
 #include <syslog.h>
-#include <sys/socket.h>
 #include <sys/stat.h>
 
-#ifdef HAVE_FCNTL_H
-# include <fcntl.h>
-#endif
-
 #ifdef HAVE_SYS_TIME_H
 # include <sys/time.h>
 #endif
+#include <time.h>
 
 /* Needed on AIX 4.1 to get fd_set and friends. */
 #ifdef HAVE_SYS_SELECT_H
 # include <sys/select.h>
 #endif
 
-#include "libinn.h"
-#include "macros.h"
-#include "nntp.h"
+#include "inn/innconf.h"
+#include "inn/messages.h"
+#include "inn/timer.h"
+#include "inn/libinn.h"
+#include "inn/nntp.h"
 
-/*
-** Syslog formats - collected together so they remain consistent
-*/
-STATIC char	STAT1[] =
-	"%s stats offered %lu accepted %lu refused %lu rejected %lu";
-STATIC char	STAT2[] = "%s times user %.3f system %.3f elapsed %.3f";
-STATIC char	CANT_CONNECT[] = "%s connect failed %s";
-STATIC char	CANT_AUTHENTICATE[] = "%s authenticate failed %s";
-STATIC char	XBATCH_FAIL[] = "%s xbatch failed %s";
-STATIC char	UNKNOWN_REPLY[] = "Unknown reply after sending batch -- %s";
-STATIC char	CANNOT_UNLINK[] = "cannot unlink %s: %m";
 /*
 **  Global variables.
 */
-STATIC BOOL		Debug = 0;
-STATIC BOOL		STATprint;
-STATIC char		*REMhost;
-STATIC double		STATbegin;
-STATIC double		STATend;
-STATIC char		*XBATCHname;
-STATIC int		FromServer;
-STATIC int		ToServer;
-STATIC sig_atomic_t	GotAlarm;
-STATIC sig_atomic_t	GotInterrupt;
-STATIC sig_atomic_t	JMPyes;
-STATIC jmp_buf		JMPwhere;
-STATIC unsigned long	STATaccepted;
-STATIC unsigned long	STAToffered;
-STATIC unsigned long	STATrefused;
-STATIC unsigned long	STATrejected;
+static bool		Debug = false;
+static bool		STATprint;
+static char		*REMhost;
+static double		STATbegin;
+static double		STATend;
+static char		*XBATCHname;
+static int		FromServer;
+static int		ToServer;
+static sig_atomic_t	GotAlarm;
+static sig_atomic_t	GotInterrupt;
+static sig_atomic_t	JMPyes;
+static jmp_buf		JMPwhere;
+static unsigned long	STATaccepted;
+static unsigned long	STAToffered;
+static unsigned long	STATrefused;
+static unsigned long	STATrejected;
+
+/*
+**  Prototypes.
+*/
+static void             ExitWithStats(int) __attribute__ ((__noreturn__));
 
 /*
 **  Send a line to the server. \r\n will be appended
 */
-STATIC BOOL
-REMwrite(fd, p)
-register int	fd;
-register char	*p;
+static bool
+REMwrite(int fd, char *p)
 {
-  register int		i;
-  register int		err;
-  register char		*dest;
-  static char		buff[NNTP_STRLEN];
+  int		i;
+  int		err;
+  char		*dest;
+  static char		buff[NNTP_MAXLEN_COMMAND];
 
   for (dest = buff, i = 0; p[i]; ) *dest++ = p[i++];
   *dest++ = '\r';
@@ -106,49 +99,45 @@ register char	*p;
   for (dest = buff, i+=2; i; dest += err, i -= err) {
     err = write(fd, dest, i);
     if (err < 0) {
-      (void)fprintf(stderr, "cant write %s to %s: %s\n",
-		    REMhost, dest, strerror(errno));
-      return FALSE;
+      syswarn("cannot write %s to %s", dest, REMhost);
+      return false;
     }
   }
   if (Debug)
-    (void)fprintf(stderr, "> %s\n", p);
+    fprintf(stderr, "> %s\n", p);
 
-  return TRUE;
+  return true;
 }
 
 /*
 **  Print transfer statistics, clean up, and exit.
 */
-STATIC NORETURN
-ExitWithStats(x)
-int			x;
+static void
+ExitWithStats(int x)
 {
   static char		QUIT[] = "quit";
-  TIMEINFO		Now;
   double		usertime;
   double		systime;
 
-  (void)REMwrite(ToServer, QUIT);
+  REMwrite(ToServer, QUIT);
 
-  (void)GetTimeInfo(&Now);
-  STATend = TIMEINFOasDOUBLE(Now);
+  STATend = TMRnow_double();
   if (GetResourceUsage(&usertime, &systime) < 0) {
     usertime = 0;
     systime = 0;
   }
 
   if (STATprint) {
-    (void)printf(STAT1,
-	REMhost, STAToffered, STATaccepted, STATrefused, STATrejected);
-    (void)printf("\n");
-    (void)printf(STAT2, REMhost, usertime, systime, STATend - STATbegin);
-    (void)printf("\n");
+      printf("%s stats offered %lu accepted %lu refused %lu rejected %lu\n",
+             REMhost, STAToffered, STATaccepted, STATrefused, STATrejected);
+      printf("%s times user %.3f system %.3f elapsed %.3f\n", REMhost,
+             usertime, systime, STATend - STATbegin);
   }
 
-  syslog(L_NOTICE, STAT1,
+  notice("%s stats offered %lu accepted %lu refused %lu rejected %lu",
 	 REMhost, STAToffered, STATaccepted, STATrefused, STATrejected);
-  syslog(L_NOTICE, STAT2, REMhost, usertime, systime, STATend - STATbegin);
+  notice("%s times user %.3f system %.3f elapsed %.3f", REMhost, usertime,
+         systime, STATend - STATbegin);
 
   exit(x);
   /* NOTREACHED */
@@ -158,9 +147,8 @@ int			x;
 /*
 **  Clean up the NNTP escapes from a line.
 */
-STATIC char *
-REMclean(buff)
-    char	*buff;
+static char *
+REMclean(char *buff)
 {
     char	*p;
 
@@ -177,34 +165,31 @@ REMclean(buff)
 /*
 **  Read a line of input, with timeout. We expect only answer lines, so
 **  we ignore \r\n-->\n mapping and the dot escape.
-**  Return TRUE if okay, *or we got interrupted.*
+**  Return true if okay, *or we got interrupted.*
 */
-STATIC BOOL
-REMread(start, size)
-char	*start;
-int	size;
+static bool
+REMread(char *start, int size)
 {
-  register char		*p;
-  register char		*h;
-  struct timeval	t;
-  FDSET			rmask;
-  int			i;
+  char *p, *h;
+  struct timeval t;
+  fd_set rmask;
+  int i;
 
   for (p = start; size; ) {
     FD_ZERO(&rmask);
     FD_SET(FromServer, &rmask);
     t.tv_sec = 10 * 60;
     t.tv_usec = 0;
-    i = select(FromServer + 1, &rmask, (FDSET *)NULL, (FDSET *)NULL, &t);
-    if (GotInterrupt) return TRUE;
+    i = select(FromServer + 1, &rmask, NULL, NULL, &t);
+    if (GotInterrupt) return true;
     if (i < 0) {
       if (errno == EINTR) continue;
-      else return FALSE;
+      else return false;
     }
-    if (i == 0 || !FD_ISSET(FromServer, &rmask)) return FALSE;
+    if (i == 0 || !FD_ISSET(FromServer, &rmask)) return false;
     i = read(FromServer, p, size-1);
-    if (GotInterrupt) return TRUE;
-    if (i <= 0) return FALSE;
+    if (GotInterrupt) return true;
+    if (i <= 0) return false;
     h = p;
     p += i;
     size -= i;
@@ -217,9 +202,9 @@ int	size;
   }
 
   if (Debug)
-    (void)fprintf(stderr, "< %s\n", start);
+    fprintf(stderr, "< %s\n", start);
 
-  return TRUE;
+  return true;
 }
 
 
@@ -227,9 +212,9 @@ int	size;
 **  Handle the interrupt.
 */
 static void
-Interrupted()
+Interrupted(void)
 {
-  (void)fprintf(stderr, "Interrupted\n");
+  warn("interrupted");
   ExitWithStats(1);
 }
 
@@ -238,92 +223,83 @@ Interrupted()
 **  Send a whole xbatch to the server. Uses the global variables
 **  REMbuffer & friends
 */
-STATIC BOOL
-REMsendxbatch(fd, buf, size)
-int fd;
-char *buf;
-int size;
+static bool
+REMsendxbatch(int fd, char *buf, int size)
 {
-  register char	*p;
+  char	*p;
   int		i;
   int		err;
 
   for (i = size, p = buf; i; p += err, i -= err) {
     err = write(fd, p, i);
     if (err < 0) {
-      (void)fprintf(stderr, "cant write xbatch to %s: %s\n",
-		    REMhost, strerror(errno));
-      return FALSE;
+      syswarn("cannot write xbatch to %s", REMhost);
+      return false;
     }
   }
   if (GotInterrupt) Interrupted();
   if (Debug)
-    (void)fprintf(stderr, "> [%d bytes of xbatch]\n", size);
+    fprintf(stderr, "> [%d bytes of xbatch]\n", size);
 
   /* What did the remote site say? */
   if (!REMread(buf, size)) {
-    (void)fprintf(stderr, "No reply after sending xbatch, %s\n",
-		  strerror(errno));
-    return FALSE;
+    syswarn("no reply after sending xbatch");
+    return false;
   }
   if (GotInterrupt) Interrupted();
   
   /* Parse the reply. */
   switch (atoi(buf)) {
   default:
-    (void)fprintf(stderr, "Unknown reply after sending batch -- %s", buf);
-    syslog(L_ERROR, UNKNOWN_REPLY, buf);
-    return FALSE;
+    warn("unknown reply after sending batch -- %s", buf);
+    return false;
     /* NOTREACHED */
     break;
-  case NNTP_RESENDIT_VAL:
-  case NNTP_GOODBYE_VAL:
-    syslog(L_NOTICE, XBATCH_FAIL, REMhost, buf);
+  case NNTP_FAIL_XBATCH:
+  case NNTP_FAIL_TERMINATING:
+  case NNTP_FAIL_ACTION:
+    notice("%s xbatch failed %s", REMhost, buf);
     STATrejected++;
-    return FALSE;
+    return false;
     /* NOTREACHED */
     break;
-  case NNTP_OK_XBATCHED_VAL:
+  case NNTP_OK_XBATCH:
     STATaccepted++;
-    if (Debug) (void)fprintf(stderr, "will unlink(%s)\n", XBATCHname);
+    if (Debug) fprintf(stderr, "will unlink(%s)\n", XBATCHname);
     if (unlink(XBATCHname)) {
       /* probably another incarantion was faster, so avoid further duplicate
        * work
        */
-      (void)fprintf(stderr, "cannot unlink %s: %s\n",
-		    XBATCHname, strerror(errno));
-      syslog(L_NOTICE, CANNOT_UNLINK, XBATCHname);
-      return FALSE;
+      syswarn("cannot unlink %s", XBATCHname);
+      return false;
     }
     break;
   }
   
   /* Article sent */
-  return TRUE;
+  return true;
 }
 
 /*
 **  Mark that we got interrupted.
 */
-STATIC SIGHANDLER
-CATCHinterrupt(s)
-    int		s;
+static void
+CATCHinterrupt(int s)
 {
-    GotInterrupt = TRUE;
+    GotInterrupt = true;
+
     /* Let two interrupts kill us. */
-    (void)xsignal(s, SIG_DFL);
+    xsignal(s, SIG_DFL);
 }
 
 
 /*
 **  Mark that the alarm went off.
 */
-/* ARGSUSED0 */
-STATIC SIGHANDLER
-CATCHalarm(s)
-    int		s;
+static void
+CATCHalarm(int s UNUSED)
 {
-    GotAlarm = TRUE;
+    GotAlarm = true;
     if (JMPyes)
 	longjmp(JMPwhere, 1);
 }
@@ -332,46 +308,48 @@ CATCHalarm(s)
 /*
 **  Print a usage message and exit.
 */
-STATIC NORETURN
-Usage()
+static void
+Usage(void)
 {
-    (void)fprintf(stderr,
-	"Usage: innxbatch [-D] [-v] [-t#] [-T#] host file ...\n");
+    warn("Usage: innxbatch [-Dv] [-t#] [-T#] host file ...");
 #ifdef FROMSTDIN
-    (void)fprintf(stderr,
-        "       innxbatch [-D] [-v] [-t#] [-T#] -i host\n");
+    warn("       innxbatch [-Dv] [-t#] [-T#] -i host");
 #endif
     exit(1);
 }
 
 
 int
-main(ac, av)
-int ac;
-char *av[];
+main(int ac, char *av[])
 {
   int			i;
-  register char		*p;
-  TIMEINFO		Now;
+  char                  *p;
   FILE			*From;
   FILE			*To;
-  char			buff[NNTP_STRLEN];
-  SIGHANDLER		(*old)();
-  unsigned int		ConnectTimeout;
-  unsigned int		TotalTimeout;
+  char			buff[NNTP_MAXLEN_COMMAND];
+  void	        	(*volatile old)(int) = NULL;
   struct stat		statbuf;
   int			fd;
   int			err;
-  char			*XBATCHbuffer = NULL;
-  int			XBATCHbuffersize = 0;
-  int			XBATCHsize;
+  char *volatile        XBATCHbuffer = NULL;
+  char **volatile       argv;
+  volatile int		XBATCHbuffersize = 0;
+  volatile int		XBATCHsize, argc;
+  volatile unsigned int	ConnectTimeout;
+  volatile unsigned int	TotalTimeout;
+
+  openlog("innxbatch", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);
+  message_program_name = "innxbatch";
+  message_handlers_warn(1, message_log_syslog_err, message_log_stderr);
+  message_handlers_die(1, message_log_syslog_err, message_log_stderr);
+  message_handlers_notice(1, message_log_syslog_notice);
 
-  (void)openlog("innxbatch", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);
   /* Set defaults. */
-  if (ReadInnConf() < 0) exit(1);
+  if (!innconf_read(NULL))
+      exit(1);
   ConnectTimeout = 0;
   TotalTimeout = 0;
-  (void)umask(NEWSUMASK);
+  umask(NEWSUMASK);
 
   /* Parse JCL. */
   while ((i = getopt(ac, av, "Dit:T:v")) != EOF)
@@ -381,11 +359,11 @@ char *av[];
       /* NOTREACHED */
       break;
     case 'D':
-      Debug++;
+      Debug = true;
       break;
 #ifdef FROMSTDIN
     case 'i':
-      FromStdin = TRUE;
+      FromStdin = true;
       break;
 #endif
     case 't':
@@ -395,7 +373,7 @@ char *av[];
       TotalTimeout = atoi(optarg);
       break;
     case 'v':
-      STATprint = TRUE;
+      STATprint = true;
       break;
     }
   ac -= optind;
@@ -407,46 +385,42 @@ char *av[];
   REMhost = av[0];
   ac--;
   av++;
+  argc = ac;
+  argv = av;
 
   /* Open a connection to the remote server. */
   if (ConnectTimeout) {
-    GotAlarm = FALSE;
+    GotAlarm = false;
     old = xsignal(SIGALRM, CATCHalarm);
-    JMPyes = TRUE;
-    if (setjmp(JMPwhere)) {
-      (void)fprintf(stderr, "Can't connect to %s, timed out\n",
-		    REMhost);
-      exit(1);
-    }
-    (void)alarm(ConnectTimeout);
+    JMPyes = true;
+    if (setjmp(JMPwhere))
+      die("cannot connect to %s: timed out", REMhost);
+    alarm(ConnectTimeout);
   }
-  if (NNTPconnect(REMhost, NNTP_PORT, &From, &To, buff) < 0 || GotAlarm) {
+  if (NNTPconnect(REMhost, NNTP_PORT, &From, &To, buff, sizeof(buff)) < 0
+      || GotAlarm) {
     i = errno;
-    (void)fprintf(stderr, "Can't connect to %s, %s\n",
-		  REMhost, buff[0] ? REMclean(buff) : strerror(errno));
     if (GotAlarm)
-      syslog(L_NOTICE, CANT_CONNECT, REMhost, "timeout");
+        warn("%s connect failed: timeout", REMhost);
     else
-      syslog(L_NOTICE, CANT_CONNECT, REMhost,
-	     buff[0] ? REMclean(buff) : strerror(i));
+        syswarn("%s connect failed: %s", REMhost,
+                buff[0] ? REMclean(buff) : strerror(i));
     exit(1);
   }
 
   if (Debug)
-    (void)fprintf(stderr, "< %s\n", REMclean(buff));
+    fprintf(stderr, "< %s\n", REMclean(buff));
   if (NNTPsendpassword(REMhost, From, To) < 0 || GotAlarm) {
     i = errno;
-    (void)fprintf(stderr, "Can't authenticate with %s, %s\n",
-		  REMhost, strerror(errno));
-    syslog(L_ERROR, CANT_AUTHENTICATE,
-	   REMhost, GotAlarm ? "timeout" : strerror(i));
+    syswarn("%s authentication failed: %s", REMhost,
+            GotAlarm ? "timeout" : strerror(i));
     /* Don't send quit; we want the remote to print a message. */
     exit(1);
   }
   if (ConnectTimeout) {
-    (void)alarm(0);
-    (void)xsignal(SIGALRM, old);
-    JMPyes = FALSE;
+    alarm(0);
+    xsignal(SIGALRM, old);
+    JMPyes = false;
   }
   
   /* We no longer need standard I/O. */
@@ -456,93 +430,83 @@ char *av[];
 #if	defined(SOL_SOCKET) && defined(SO_SNDBUF) && defined(SO_RCVBUF)
   i = 24 * 1024;
   if (setsockopt(ToServer, SOL_SOCKET, SO_SNDBUF, (char *)&i, sizeof i) < 0)
-    (void)perror("cant setsockopt(SNDBUF)");
+    syswarn("cant setsockopt(SNDBUF)");
   if (setsockopt(FromServer, SOL_SOCKET, SO_RCVBUF, (char *)&i, sizeof i) < 0)
-    (void)perror("cant setsockopt(RCVBUF)");
+    syswarn("cant setsockopt(RCVBUF)");
 #endif	/* defined(SOL_SOCKET) && defined(SO_SNDBUF) && defined(SO_RCVBUF) */
 
-  GotInterrupt = FALSE;
-  GotAlarm = FALSE;
+  GotInterrupt = false;
+  GotAlarm = false;
 
   /* Set up signal handlers. */
-  (void)xsignal(SIGHUP, CATCHinterrupt);
-  (void)xsignal(SIGINT, CATCHinterrupt);
-  (void)xsignal(SIGTERM, CATCHinterrupt);
-  (void)xsignal(SIGPIPE, SIG_IGN);
+  xsignal(SIGHUP, CATCHinterrupt);
+  xsignal(SIGINT, CATCHinterrupt);
+  xsignal(SIGTERM, CATCHinterrupt);
+  xsignal(SIGPIPE, SIG_IGN);
   if (TotalTimeout) {
-    (void)xsignal(SIGALRM, CATCHalarm);
-    (void)alarm(TotalTimeout);
+    xsignal(SIGALRM, CATCHalarm);
+    alarm(TotalTimeout);
   }
 
   /* Start timing. */
-  if (GetTimeInfo(&Now) < 0) {
-    (void)fprintf(stderr, "Can't get time, %s\n", strerror(errno));
-    exit(1);
-  }
-  STATbegin = TIMEINFOasDOUBLE(Now);
-
+  STATbegin = TMRnow_double();
 
   /* main loop over all specified files */
-  for (XBATCHname = *av; ac && (XBATCHname = *av); av++, ac--) {
+  for (XBATCHname = *argv; argc && (XBATCHname = *argv); argv++, argc--) {
 
-    if (Debug) (void)fprintf(stderr, "will work on %s\n", XBATCHname);
+    if (Debug) fprintf(stderr, "will work on %s\n", XBATCHname);
 
     if (GotAlarm) {
-      (void)fprintf(stderr, "Timed out\n");
+      warn("timed out");
       ExitWithStats(1);
     }
     if (GotInterrupt) Interrupted();
 
     if ((fd = open(XBATCHname, O_RDONLY, 0)) < 0) {
-      (void)fprintf(stderr, "Can't open \"%s\", %s - skipping it\n",
-		    XBATCHname, strerror(errno));
+      syswarn("cannot open %s, skipping", XBATCHname);
       continue;
     }
 
     if (fstat(fd, &statbuf)) {
-      (void)fprintf(stderr, "Can't stat \"%s\", %s - skipping it\n",
-		    XBATCHname, strerror(errno));
-      (void)close(i);
+      syswarn("cannot stat %s, skipping", XBATCHname);
+      close(i);
       continue;
     }
 
     XBATCHsize = statbuf.st_size;
     if (XBATCHsize == 0) {
-      (void)fprintf(stderr, "Batch file \"%s\" is zero length, - skipping it\n",
-		    XBATCHname);
-      (void)close(i);
-      (void)unlink(XBATCHname);
+      warn("batch file %s is zero length, skipping", XBATCHname);
+      close(i);
+      unlink(XBATCHname);
       continue;
     } else if (XBATCHsize > XBATCHbuffersize) {
       XBATCHbuffersize = XBATCHsize;
-      if (XBATCHbuffer) DISPOSE(XBATCHbuffer);
-      XBATCHbuffer = NEW(char, XBATCHsize);
+      if (XBATCHbuffer) free(XBATCHbuffer);
+      XBATCHbuffer = xmalloc(XBATCHsize);
     }
 
+    err = 0; /* stupid compiler */
     for (i = XBATCHsize, p = XBATCHbuffer; i; i -= err, p+= err) {
       err = read(fd, p, i);
       if (err < 0) {
-	(void)fprintf(stderr, "error reading %s: %s - skipping it\n",
-		      XBATCHname, strerror(errno));
+        syswarn("error reading %s, skipping", XBATCHname);
 	break;
       } else if (0 == err) {
-	(void)fprintf(stderr, "unexpected EOF reading %s: %s - truncated\n",
-		      XBATCHname, strerror(errno));
+        syswarn("unexpected EOF reading %s, truncated", XBATCHname);
 	XBATCHsize = p - XBATCHbuffer;
 	break;
       }
     }
-    (void)close(fd);
+    close(fd);
     if (err < 0)
       continue;
 
     if (GotInterrupt) Interrupted();
 
     /* Offer the xbatch. */
-    (void)sprintf(buff, "xbatch %d", XBATCHsize);
+    snprintf(buff, sizeof(buff), "XBATCH %d", XBATCHsize);
     if (!REMwrite(ToServer, buff)) {
-      (void)fprintf(stderr, "Can't offer xbatch to %s, %s\n",
-		    REMhost, strerror(errno));
+      syswarn("cannot offer xbatch to %s", REMhost);
       ExitWithStats(1);
     }
     STAToffered++;
@@ -550,8 +514,7 @@ char *av[];
 
     /* Does he want it? */
     if (!REMread(buff, (int)sizeof buff)) {
-      (void)fprintf(stderr, "No reply to XBATCH %d from %s, %s\n",
-		    XBATCHsize, REMhost, strerror(errno));
+      syswarn("no reply to XBATCH %d from %s", XBATCHsize, REMhost);
       ExitWithStats(1);
     }
     if (GotInterrupt) Interrupted();
@@ -559,26 +522,25 @@ char *av[];
     /* Parse the reply. */
     switch (atoi(buff)) {
     default:
-      (void)fprintf(stderr, "Unknown reply to \"%s\" -- %s", XBATCHname, buff);
+      warn("unknown reply to %s -- %s", XBATCHname, buff);
       ExitWithStats(1);
       /* NOTREACHED */
       break;
-    case NNTP_RESENDIT_VAL:
-    case NNTP_GOODBYE_VAL:
+    case NNTP_FAIL_XBATCH:
+    case NNTP_FAIL_TERMINATING:
+    case NNTP_FAIL_ACTION:
       /* Most likely out of space -- no point in continuing. */
-      syslog(L_NOTICE, XBATCH_FAIL, REMhost, buff);
+      notice("%s xbatch failed %s", REMhost, buff);
       ExitWithStats(1);
       /* NOTREACHED */
-    case NNTP_CONT_XBATCH_VAL:
+    case NNTP_CONT_XBATCH:
       if (!REMsendxbatch(ToServer, XBATCHbuffer, XBATCHsize))
 	ExitWithStats(1);
       /* NOTREACHED */
       break;
-    case NNTP_SYNTAX_VAL:
-    case NNTP_BAD_COMMAND_VAL:
-      (void)fprintf(stderr, "Server %s seems not understand XBATCH: %s\n",
-		    REMhost, buff);
-      syslog(L_FATAL, XBATCH_FAIL, REMhost, buff);
+    case NNTP_ERR_SYNTAX:
+    case NNTP_ERR_COMMAND:
+      warn("%s xbatch failed %s", REMhost, buff);
       break;
     }
   }
