@@ -1,4 +1,4 @@
-/*  $Id: innxmit.c 9876 2015-05-23 13:11:16Z iulius $
+/*  $Id: innxmit.c 10153 2017-06-05 12:28:01Z iulius $
 **
 **  Transmit articles to remote site.
 **  Modified for NNTP streaming: 1996-01-03 Jerry Aguirre.
@@ -7,7 +7,6 @@
 #include "config.h"
 #include "clibrary.h"
 #include "portable/socket.h"
-#include "portable/time.h"
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -17,6 +16,11 @@
 #include <sys/stat.h>
 #include <sys/uio.h>
 
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+#include <time.h>
+
 /*
 **  Needed on AIX 4.1 to get fd_set and friends.
 */
@@ -91,19 +95,18 @@ static int logRejects = false ;  /* syslog the 437 responses. */
 /*
 **  Syslog formats - collected together so they remain consistent.
 */
-static char	STAT1[] =
-	"%s stats offered %lu accepted %lu refused %lu rejected %lu missing %lu accsize %.0f rejsize %.0f";
-static char	STAT2[] = "%s times user %.3f system %.3f elapsed %.3f";
-static char	GOT_BADCOMMAND[] = "%s rejected %s %s";
-static char	REJECTED[] = "%s rejected %s (%s) %s";
-static char	REJ_STREAM[] = "%s rejected (%s) %s";
-static char	CANT_CONNECT[] = "%s connect failed %s";
-static char	CANT_AUTHENTICATE[] = "%s authenticate failed %s";
-static char	IHAVE_FAIL[] = "%s ihave failed %s";
+#define STAT1 "%s stats offered %lu accepted %lu refused %lu rejected %lu missing %lu accsize %.0f rejsize %.0f"
+#define STAT2 "%s times user %.3f system %.3f elapsed %.3f"
+#define GOT_BADCOMMAND "%s rejected %s %s"
+#define REJECTED "%s rejected %s (%s) %s"
+#define REJ_STREAM "%s rejected (%s) %s"
+#define CANT_CONNECT "%s connect failed %s"
+#define CANT_AUTHENTICATE "%s authenticate failed %s"
+#define IHAVE_FAIL "%s ihave failed %s"
 
-static char	CANT_FINDIT[] = "%s can't find %s";
-static char	CANT_PARSEIT[] = "%s can't parse ID %s";
-static char	UNEXPECTED[] = "%s unexpected response code %s";
+#define CANT_FINDIT "%s can't find %s"
+#define CANT_PARSEIT "%s can't parse ID %s"
+#define UNEXPECTED "%s unexpected response code %s"
 
 /*
 **  Global variables.
@@ -145,6 +148,9 @@ static double		STATrejectedsize;
 */
 static ARTHANDLE *article_open(const char *path, const char *id);
 static void article_free(ARTHANDLE *);
+static void Usage(void) __attribute__ ((__noreturn__));
+static void RequeueRestAndExit(char *, char *) __attribute__ ((__noreturn__));
+static void ExitWithStats(int) __attribute__ ((__noreturn__));
 
 
 /*
@@ -1002,7 +1008,6 @@ article_free(ARTHANDLE *article)
 
 
 int main(int ac, char *av[]) {
-    static char		SKIPPING[] = "Skipping \"%s\" --%s?\n";
     int	                i;
     char	        *p;
     ARTHANDLE		*art;
@@ -1335,7 +1340,7 @@ int main(int ac, char *av[]) {
 	/* Get the Message-ID from the article if we need to. */
 	if (MessageID == NULL) {
 	    if ((MessageID = GetMessageID(art)) == NULL) {
-                warn(SKIPPING, Article, "no message ID");
+                warn("Skipping \"%s\" -- %s?\n", Article, "no message ID");
                 article_free(art);
 		continue;
 	    }
