@@ -1,4 +1,4 @@
-/*  $Id: nntpget.c,v 1.33 2003/01/19 01:15:26 rra Exp $
+/*  $Id: nntpget.c 10153 2017-06-05 12:28:01Z iulius $
 **
 **  Connect to a remote site, and get news from it to offer to our local
 **  server.  Read list on stdin, or get it via NEWNEWS command.  Writes
@@ -8,11 +8,15 @@
 #include "config.h"
 #include "clibrary.h"
 #include "portable/socket.h"
-#include "portable/time.h"
 #include <errno.h>
 #include <sys/stat.h>
 #include <sys/uio.h>
 
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+#include <time.h>
+
 /* Needed on AIX 4.1 to get fd_set and friends. */
 #ifdef HAVE_SYS_SELECT_H
 # include <sys/select.h>
@@ -21,9 +25,9 @@
 #include "inn/history.h"
 #include "inn/innconf.h"
 #include "inn/messages.h"
-#include "libinn.h"
-#include "nntp.h"
-#include "paths.h"
+#include "inn/libinn.h"
+#include "inn/nntp.h"
+#include "inn/paths.h"
 
 /*
 **  All information about a site we are connected to.
@@ -43,14 +47,20 @@ typedef struct _SITE {
 */
 static struct iovec	SITEvec[2];
 static char		SITEv1[] = "\r\n";
-static char		READER[] = "mode reader";
+static char		READER[] = "MODE READER";
 static unsigned long	STATgot;
 static unsigned long	STAToffered;
 static unsigned long	STATsent;
 static unsigned long	STATrejected;
 static struct history	*History;
 
-
+
+/*
+**  Prototypes.
+*/
+static void             Usage(const char *) __attribute__ ((__noreturn__));
+
+
 
 /*
 **  Read a line of input, with timeout.
@@ -65,7 +75,7 @@ SITEread(SITE *sp, char *start)
     int			i;
     char		c;
 
-    for (p = start, end = &start[NNTP_STRLEN - 1]; ; ) {
+    for (p = start, end = &start[NNTP_MAXLEN_COMMAND - 1]; ; ) {
 	if (sp->Count == 0) {
 	    /* Fill the buffer. */
     Again:
@@ -129,12 +139,12 @@ SITEconnect(char *host)
 
     /* Connect and identify ourselves. */
     if (host)
-	i = NNTPconnect(host, NNTP_PORT, &From, &To, (char *)NULL);
+	i = NNTPconnect(host, NNTP_PORT, &From, &To, NULL, 0);
     else {
 	host = innconf->server;
         if (host == NULL)
             die("no server specified and server not set in inn.conf");
-	i = NNTPlocalopen(&From, &To, (char *)NULL);
+	i = NNTPlocalopen(&From, &To, NULL, 0);
     }
     if (i < 0)
         sysdie("cannot connect to %s", host);
@@ -159,9 +169,9 @@ SITEconnect(char *host)
 static void
 SITEquit(SITE *sp)
 {
-    char	buff[NNTP_STRLEN];
+    char	buff[NNTP_MAXLEN_COMMAND];
 
-    SITEwrite(sp, "quit", 4);
+    SITEwrite(sp, "QUIT", 4);
     SITEread(sp, buff);
 }
 
@@ -186,8 +196,8 @@ Usage(const char *p)
 int
 main(int ac, char *av[])
 {
-    char	buff[NNTP_STRLEN];
-    char	mesgid[NNTP_STRLEN];
+    char	buff[NNTP_MAXLEN_COMMAND];
+    char	mesgid[NNTP_MAXLEN_COMMAND];
     char	tbuff[SMBUF];
     char	*msgidfile = NULL;
     int         msgidfd;
@@ -239,10 +249,9 @@ main(int ac, char *av[])
 	    if (stat(optarg, &Sb) < 0)
                 sysdie("cannot stat %s", optarg);
 	    gt = gmtime(&Sb.st_mtime);
-	    /* Y2K: NNTP Spec currently allows only two digit years. */
-	    snprintf(tbuff, sizeof(tbuff), "%02d%02d%02d %02d%02d%02d GMT",
-		    gt->tm_year % 100, gt->tm_mon + 1, gt->tm_mday,
-		    gt->tm_hour, gt->tm_min, gt->tm_sec);
+            snprintf(tbuff, sizeof(tbuff), "%04d%02d%02d %02d%02d%02d GMT",
+                     gt->tm_year + 1900, gt->tm_mon + 1, gt->tm_mday,
+                     gt->tm_hour, gt->tm_min, gt->tm_sec);
 	    Since = tbuff;
 	    break;
 	case 'n':
@@ -250,7 +259,7 @@ main(int ac, char *av[])
 	    break;
 	case 'o':
 	    /* Open the history file. */
-            path = concatpath(innconf->pathdb, _PATH_HISTORY);
+            path = concatpath(innconf->pathdb, INN_PATH_HISTORY);
 	    History = HISopen(path, innconf->hismethod, HIS_RDONLY);
 	    if (!History)
                 sysdie("cannot open history");
@@ -359,25 +368,25 @@ main(int ac, char *av[])
 	if (Offer) {
 	    /* See if the local server wants it. */
 	    STAToffered++;
-	    snprintf(buff, sizeof(buff), "ihave %s", mesgid);
+	    snprintf(buff, sizeof(buff), "IHAVE %s", mesgid);
 	    if (!SITEwrite(Local, buff, (int)strlen(buff))
 	     || !SITEread(Local, buff)) {
                 syswarn("cannot offer %s", mesgid);
 		break;
 	    }
-	    if (atoi(buff) != NNTP_SENDIT_VAL)
+	    if (atoi(buff) != NNTP_CONT_IHAVE)
 		continue;
 	}
 
 	/* Try to get the article. */
-	snprintf(buff, sizeof(buff), "article %s", mesgid);
+	snprintf(buff, sizeof(buff), "ARTICLE %s", mesgid);
 	if (!SITEwrite(Remote, buff, (int)strlen(buff))
 	 || !SITEread(Remote, buff)) {
             syswarn("cannot get %s", mesgid);
 	    printf("%s\n", mesgid);
 	    break;
 	}
-	if (atoi(buff) != NNTP_ARTICLE_FOLLOWS_VAL) {
+	if (atoi(buff) != NNTP_OK_ARTICLE) {
           if (Offer) {
               SITEwrite(Local, ".", 1);
               if (!SITEread(Local, buff)) {
@@ -424,9 +433,9 @@ main(int ac, char *av[])
 		break;
 	    }
 	    i = atoi(buff);
-	    if (i == NNTP_TOOKIT_VAL)
+	    if (i == NNTP_OK_IHAVE)
 		continue;
-	    if (i == NNTP_RESENDIT_VAL) {
+	    if (i == NNTP_FAIL_IHAVE_DEFER) {
 		printf("%s\n", mesgid);
 		break;
 	    }
@@ -458,7 +467,7 @@ main(int ac, char *av[])
     if (Update) {
 	if ((F = fopen(Update, "w")) == NULL)
             sysdie("cannot update %s", Update);
-	fprintf(F, "got %ld offered %ld sent %ld rejected %ld\n",
+	fprintf(F, "got %lu offered %lu sent %lu rejected %lu\n",
 		STATgot, STAToffered, STATsent, STATrejected); 
 	if (ferror(F) || fclose(F) == EOF)
             sysdie("cannot update %s", Update);
