@@ -1,4 +1,4 @@
-/*  $Id: shlock.c 8174 2008-11-18 15:11:03Z iulius $
+/*  $Id: shlock.c 10153 2017-06-05 12:28:01Z iulius $
 **
 **  Produce reliable locks for shell scripts, by Peter Honeyman as told
 **  to Rich $alz.
@@ -18,15 +18,16 @@
 
 static bool BinaryLock;
 static bool JustChecking;
+static void Usage(void) __attribute__ ((__noreturn__));
 
-static char CANTUNLINK[] = "shlock: Can't unlink \"%s\", %s";
-static char CANTOPEN[] = "shlock: Can't open \"%s\", %s";
-static char CANTCHDIR[] = "shlock: Can't chdir to \"%s\", %s";
-static char CANTWRITEPID[] = "shlock: Can't write PID to \"%s\", %s";
-static char CANTCLOSE[] = "shlock: Can't close \"%s\", %s";
-static char CANTLINK[] = "shlock: Can't link \"%s\" to \"%s\", %s";
-static char STALELOCK[] = "shlock: Stale lockfile detected \"%s\", please remove";
-static char CANTSTAT[] = "shlock: Can't stat \"%s\", %s";
+#define CANTUNLINK "shlock: Can't unlink \"%s\", %s"
+#define CANTOPEN "shlock: Can't open \"%s\", %s"
+#define CANTCHDIR "shlock: Can't chdir to \"%s\", %s"
+#define CANTWRITEPID "shlock: Can't write PID to \"%s\", %s"
+#define CANTCLOSE "shlock: Can't close \"%s\", %s"
+#define CANTLINK "shlock: Can't link \"%s\" to \"%s\", %s"
+#define STALELOCK "shlock: Stale lockfile detected \"%s\", please remove"
+#define CANTSTAT "shlock: Can't stat \"%s\", %s"
 
 /* After this time, we start to complain about an invalid locked lockfile. */
 #define LINK_TIMEOUT 30
@@ -172,7 +173,6 @@ main(int ac, char *av[])
   }
 
   ac -= optind;
-  av += optind;
   if (ac || pid == 0 || name == NULL) {
     Usage();
   }
