@@ -1,4 +1,4 @@
-/*  $Id: innd.c 8886 2010-01-16 16:07:19Z iulius $
+/*  $Id: innd.c 10263 2018-03-10 14:51:53Z iulius $
 **
 **  Variable definitions, miscellany, and main().
 */
@@ -31,6 +31,9 @@ bool		ThrottledbyIOError = false;
 
 static char	*PID = NULL;
 
+/* Default values for the syntaxchecks parameter in inn.conf. */
+bool laxmid = false;
+
 /* Signal handling.  If we receive a signal that should kill the server,
    killer_signal is set to the signal number that we received.  This isn't
    what indicates that we should terminate; that's the separate global
@@ -58,6 +61,7 @@ FILE *Errlog = NULL;
 static void             catch_terminate(int sig);
 static void             xmalloc_abort(const char *what, size_t size,
                                       const char *file, int line);
+static void             Usage(void) __attribute__ ((__noreturn__));
 
 /* header table initialization */
 #define ARTHEADERINIT(name, type) {name, type, sizeof(name) - 1}
@@ -75,7 +79,7 @@ const ARTHEADER ARTheaders[] = {
 /* #define HDR__EXPIRES				4 */
   ARTHEADERINIT("From",			HTreq),
 /* #define HDR__FROM				5 */
-  ARTHEADERINIT("Lines",		HTstd),
+  ARTHEADERINIT("Lines",		HTobs),
 /* #define HDR__LINES				6 */
   ARTHEADERINIT("Message-ID",		HTreq),
 /* #define HDR__MESSAGE_ID			7 */
@@ -91,7 +95,7 @@ const ARTHEADER ARTheaders[] = {
 /* #define HDR__SUBJECT				12 */
   ARTHEADERINIT("Supersedes",		HTstd),
 /* #define HDR__SUPERSEDES			13 */
-  ARTHEADERINIT("Bytes",		HTstd),
+  ARTHEADERINIT("Bytes",		HTobs),
 /* #define HDR__BYTES				14 */
   ARTHEADERINIT("Also-Control",		HTobs),
 /* #define HDR__ALSOCONTROL			15 */
@@ -101,7 +105,7 @@ const ARTHEADER ARTheaders[] = {
 /* #define HDR__XREF				17 */
   ARTHEADERINIT("Keywords",		HTstd),
 /* #define HDR__KEYWORDS			18 */
-  ARTHEADERINIT("X-Trace",		HTstd),
+  ARTHEADERINIT("X-Trace",		HTobs),
 /* #define HDR__XTRACE				19 */
   ARTHEADERINIT("Date-Received",	HTobs),
 /* #define HDR__DATERECEIVED			20 */
@@ -113,7 +117,7 @@ const ARTHEADER ARTheaders[] = {
 /* #define HDR__RECEIVED			23 */
   ARTHEADERINIT("Relay-Version",	HTobs),
 /* #define HDR__RELAYVERSION			24 */
-  ARTHEADERINIT("NNTP-Posting-Host",	HTstd),
+  ARTHEADERINIT("NNTP-Posting-Host",	HTobs),
 /* #define HDR__NNTPPOSTINGHOST			25 */
   ARTHEADERINIT("Followup-To",		HTstd),
 /* #define HDR__FOLLOWUPTO			26 */
@@ -157,7 +161,7 @@ const ARTHEADER ARTheaders[] = {
 /* #define HDR__ORIGINATOR			45 */
   ARTHEADERINIT("X-Auth",		HTstd),
 /* #define HDR__X_AUTH				46 */
-  ARTHEADERINIT("X-Complaints-To",	HTstd),
+  ARTHEADERINIT("X-Complaints-To",	HTobs),
 /* #define HDR__X_COMPLAINTS_TO			47 */
   ARTHEADERINIT("X-Face",		HTstd),
 /* #define HDR__X_FACE				48 */
@@ -169,7 +173,7 @@ const ARTHEADER ARTheaders[] = {
 /* #define HDR__X_MODBOT			51 */
   ARTHEADERINIT("X-Modtrace",		HTstd),
 /* #define HDR__X_MODTRACE			52 */
-  ARTHEADERINIT("X-No-Archive",		HTstd),
+  ARTHEADERINIT("X-No-Archive",		HTobs),
 /* #define HDR__X_NO_ARCHIVE			53 */
   ARTHEADERINIT("X-Original-Trace",	HTstd),
 /* #define HDR__X_ORIGINAL_TRACE		54 */
@@ -193,7 +197,7 @@ const ARTHEADER ARTheaders[] = {
 /* #define HDR__IN_REPLY_TO			63 */
   ARTHEADERINIT("Injection-Date",	HTstd),
 /* #define HDR__INJECTION_DATE			64 */
-  ARTHEADERINIT("NNTP-Posting-Date",    HTstd),
+  ARTHEADERINIT("NNTP-Posting-Date",    HTobs),
 /* #define HDR__NNTP_POSTING_DATE               65 */
   ARTHEADERINIT("X-User-ID",            HTstd),
 /* #define HDR__X_USER_ID                       66 */
@@ -203,7 +207,7 @@ const ARTHEADER ARTheaders[] = {
 /* #define HDR__X_ORIGINAL_NNTP_POSTING_HOST    68 */
   ARTHEADERINIT("Original-Sender",      HTstd),
 /* #define HDR__ORIGINAL_SENDER                 69 */
-  ARTHEADERINIT("NNTP-Posting-Path",    HTstd),
+  ARTHEADERINIT("NNTP-Posting-Path",    HTobs),
 /* #define HDR__NNTP_POSTING_PATH               70 */
   ARTHEADERINIT("Archive",              HTstd),
 /* #define HDR__ARCHIVE                         71 */
@@ -211,10 +215,12 @@ const ARTHEADER ARTheaders[] = {
 /* #define HDR__ARCHIVED_AT                     72 */
   ARTHEADERINIT("Summary",              HTstd),
 /* #define HDR__SUMMARY                         73 */
-  ARTHEADERINIT("Comments",             HTstd)
+  ARTHEADERINIT("Comments",             HTstd),
 /* #define HDR__COMMENTS                        74 */
+  ARTHEADERINIT("Jabber-ID",            HTstd),
+/* #define HDR__JABBER_ID                       75 */
 };
-/* #define MAX_ARTHEADER                        75 */
+/* #define MAX_ARTHEADER                        76 */
 
 
 /*
@@ -351,19 +357,22 @@ main(int ac, char *av[])
     bool flag;
     static char		WHEN[] = "PID file";
     int			i;
+    size_t              j;
     char		buff[SMBUF];
     FILE		*F;
     bool		ShouldFork;
     bool		ShouldRenumber;
     bool		ShouldSyntaxCheck;
-    bool		filter = true;
     pid_t		pid;
-#if	defined(_DEBUG_MALLOC_INC)
+#if defined(_DEBUG_MALLOC_INC)
     union malloptarg	m;
 #endif	/* defined(_DEBUG_MALLOC_INC) */
 #if DO_PERL
     char                *path1, *path2;
 #endif
+#if defined(DO_PERL) || defined(DO_PYTHON)
+    bool                filter = true;
+#endif
 
     /* Set up the pathname, first thing, and teach our error handlers about
        the name of the program. */
@@ -385,6 +394,8 @@ main(int ac, char *av[])
        root, switch to running as the news user. */
     ensure_news_user_grp(true, true);
 
+    xsignal_enable_masking();
+
     /* Handle malloc debugging. */
 #if	defined(_DEBUG_MALLOC_INC)
     m.i = M_HANDLE_ABORT;
@@ -411,7 +422,7 @@ main(int ac, char *av[])
 
     /* Parse JCL. */
     CCcopyargv(av);
-    while ((i = getopt(ac, av, "4:6:ac:CdfH:i:l:m:n:No:P:rst:T:uX:")) != EOF)
+    while ((i = getopt(ac, av, "4:6:ac:CdfH:i:l:m:n:No:P:rsSt:T:uX:")) != EOF)
 	switch (i) {
 	default:
 	    Usage();
@@ -466,7 +477,9 @@ main(int ac, char *av[])
                                     "ed from the command line", (char *) 0);
 	    break;
 	case 'N':
+#if defined(DO_PERL) || defined(DO_PYTHON)
 	    filter = false;
+#endif
 	    break;
 	case 'n':
 	    switch (*optarg) {
@@ -489,6 +502,10 @@ main(int ac, char *av[])
 	case 's':
 	    ShouldSyntaxCheck = true;
 	    break;
+        case 'S':
+            RCreadlist();
+            exit(0);
+            break;
 	case 't':
 	    TimeOut.tv_sec = atol(optarg);
 	    break;
@@ -508,7 +525,7 @@ main(int ac, char *av[])
     ac -= optind;
     if (ac != 0)
 	Usage();
-    if (ModeReason && !innconf->readerswhenstopped)
+    if (ModeReason != NULL && !innconf->readerswhenstopped)
 	NNRPReason = xstrdup(ModeReason);
 
     if (ShouldSyntaxCheck) {
@@ -518,6 +535,24 @@ main(int ac, char *av[])
 	exit(1);
     }
 
+    /* Initialize the checks to perform or not on article syntax. */
+    if ((innconf->syntaxchecks != NULL) && (innconf->syntaxchecks->count > 0)) {
+        for (j = 0; j < innconf->syntaxchecks->count; j++) {
+            if (innconf->syntaxchecks->strings[j] != NULL) {
+                if (strcmp(innconf->syntaxchecks->strings[j], "laxmid") == 0) {
+                    laxmid = true;
+                } else if (strcmp(innconf->syntaxchecks->strings[j],
+                                  "no-laxmid") == 0) {
+                    laxmid = false;
+                } else {
+                    syslog(L_NOTICE, "Unknown \"%s\" value in syntaxchecks "
+                           "parameter in inn.conf",
+                           innconf->syntaxchecks->strings[j]);
+                }
+            }
+        }
+    }
+
     /* Get the Path entry. */
     if (innconf->pathhost == NULL) {
 	syslog(L_FATAL, "%s No pathhost set", LogName);
@@ -592,19 +627,34 @@ main(int ac, char *av[])
         die("SERVER cant open overview method");
 
     /* Attempt to increase the number of open file descriptors. */
-    if (innconf->rlimitnofile > 0)
+    if (innconf->rlimitnofile > 0) {
         if (setfdlimit(innconf->rlimitnofile) < 0)
             syswarn("SERVER cant set file descriptor limit");
+    }
 
     /* Get number of open channels. */
     i = getfdlimit();
     if (i < 0)
         sysdie("SERVER cant get file descriptor limit");
 
+#ifdef FD_SETSIZE
+    if (FD_SETSIZE > 0 && (unsigned) i >= FD_SETSIZE) {
+        /* Only log a warning if rlimitnofile has been set
+         * to a value different than the default setting of letting
+         * the system set the number of file descriptors. */
+        if (innconf->rlimitnofile > 0) {
+            syslog(LOG_WARNING, "%s number of descriptors (%d) exceeding or equaling FD_SETSIZE (%d)",
+                   LogName, i, FD_SETSIZE);
+        }
+        i = FD_SETSIZE-1;
+    }
+#endif
+
     /* There is no file descriptor limit on some hosts; for those, cap at
        MaxOutgoing plus maxconnections plus 20, or 5000, whichever is larger. 
        Otherwise, we use insane amounts of memory for the channel table.
-       FIXME: Get rid of this hard-coded constant. */
+       FIXME:  Get rid of this hard-coded constant.
+       (TODO:  Consider implementing libevent.) */
     if (i > 5000) {
         unsigned long max;
 
