@@ -1,4 +1,4 @@
-/*  $Id: nc.c 9399 2012-03-04 15:08:17Z iulius $
+/*  $Id: nc.c 10149 2017-06-05 12:24:58Z iulius $
 **
 **  Routines for the NNTP channel.  Other channels get the descriptors which
 **  we turn into NNTP channels, and over which we speak NNTP.
@@ -15,6 +15,8 @@
 #define BAD_COMMAND_COUNT	10
 
 
+extern bool laxmid;
+
 /*
 **  An entry in the dispatch table.  The name, and implementing function,
 **  of every command we support.
@@ -88,6 +90,9 @@ static NCDISPATCH NCcommands[] = {
        READER command. */
     COMMAND_READER("ARTICLE"),
     COMMAND_READER("BODY"),
+#if defined(HAVE_ZLIB)
+    COMMAND_READER("COMPRESS"),
+#endif /* HAVE_ZLIB */
     COMMAND_READER("DATE"),
     COMMAND_READER("GROUP"),
     COMMAND_READER("HDR"),
@@ -98,7 +103,7 @@ static NCDISPATCH NCcommands[] = {
     COMMAND_READER("NEXT"),
     COMMAND_READER("OVER"),
     COMMAND_READER("POST"),
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
     COMMAND_READER("STARTTLS"),
 #endif
     COMMAND_READER("XGTITLE"),
@@ -355,7 +360,7 @@ NChead(CHANNEL *cp)
         return;
     }
 
-    if (!IsValidMessageID(cp->av[1], true)) {
+    if (!IsValidMessageID(cp->av[1], true, laxmid)) {
         xasprintf(&buff, "%d Syntax error in message-ID", NNTP_ERR_SYNTAX);
         NCwritereply(cp, buff);
         free(buff);
@@ -421,7 +426,7 @@ NCstat(CHANNEL *cp)
         return;
     }
 
-    if (!IsValidMessageID(cp->av[1], true)) {
+    if (!IsValidMessageID(cp->av[1], true, laxmid)) {
         xasprintf(&buff, "%d Syntax error in message-ID", NNTP_ERR_SYNTAX);
         NCwritereply(cp, buff);
         free(buff);
@@ -503,9 +508,6 @@ NCauthinfo(CHANNEL *cp)
     }
 
     /* AUTHINFO PASS cannot be sent before AUTHINFO USER. */
-    /* Enforced only in INN 2.6.0, for backwards compatibility.
-     * [INN 2.5.2 implements RFC 3977 + RFC 4643 + RFC 4644 in innd.] */
-    /*
     if (!cp->HasSentUsername) {
         xasprintf(&buff, "%d Authentication commands issued out of sequence",
                   NNTP_FAIL_AUTHINFO_REJECT);
@@ -513,7 +515,6 @@ NCauthinfo(CHANNEL *cp)
         free(buff);
         return;
     }
-    */
 
     /* Got the password -- is it okay? */
     if (!RCauthorized(cp, cp->av[2])) {
@@ -623,6 +624,11 @@ NCcapabilities(CHANNEL *cp)
         WCHANappend(cp, NCterm, strlen(NCterm));
     }
 
+    if (cp->IsAuthenticated) {
+        WCHANappend(cp, "XBATCH", 6);
+        WCHANappend(cp, NCterm, strlen(NCterm));
+    }
+
     NCwritereply(cp, NCdot);
 }
 
@@ -643,7 +649,7 @@ NCihave(CHANNEL *cp)
     cp->Ihave++;
     cp->Start = cp->Next;
 
-    if (!IsValidMessageID(cp->av[1], false)) {
+    if (!IsValidMessageID(cp->av[1], false, laxmid)) {
         /* Return 435 here instead of 501 for compatibility reasons. */
         xasprintf(&buff, "%d Syntax error in message-ID", NNTP_FAIL_IHAVE_REFUSE);
         NCwritereply(cp, buff);
@@ -1092,7 +1098,6 @@ NCproc(CHANNEL *cp)
   bool          validcommandtoolong;
   int           syntaxerrorcode = NNTP_ERR_SYNTAX;
 
-  readmore = movedata = false;
   if (Tracing || cp->Tracing)
     syslog(L_TRACE, "%s NCproc Used=%lu", CHANname(cp),
            (unsigned long) cp->In.used);
@@ -1272,7 +1277,12 @@ NCproc(CHANNEL *cp)
           NCwritereply(cp, buff);
           break;
         }
-        snprintf(buff, sizeof(buff), "%d What?", NNTP_ERR_COMMAND);
+        if (strcasecmp(cp->av[0], "XYZZY") == 0) {
+            /* Acknowledge the magic word from the Colossal Cave Adventure computer game. */
+            snprintf(buff, sizeof(buff), "%d Nothing happens", NNTP_ERR_COMMAND);
+        } else {
+            snprintf(buff, sizeof(buff), "%d What?", NNTP_ERR_COMMAND);
+        }
 	NCwritereply(cp, buff);
 	cp->Start = cp->Next;
 
@@ -1608,7 +1618,6 @@ NCproc(CHANNEL *cp)
 
     if (movedata) { /* move data rather than extend buffer */
       TMRstart(TMR_DATAMOVE);
-      movedata = false;
       if (cp->Start > 0)
 	memmove(bp->data, &bp->data[cp->Start], bp->used - cp->Start);
       bp->used -= cp->Start;
@@ -1629,9 +1638,10 @@ NCproc(CHANNEL *cp)
       cp->Start = 0;
       TMRstop(TMR_DATAMOVE);
     }
-    if (readmore)
-      /* need to read more */
-      break;
+    if (readmore) {
+        /* need to read more */
+        break;
+    }
   }
 }
 
@@ -1808,7 +1818,7 @@ NCcheck(CHANNEL *cp)
             cp->Sendid.size = MED_BUFFER;
         cp->Sendid.data = xmalloc(cp->Sendid.size);
     }
-    if (!IsValidMessageID(cp->av[1], false)) {
+    if (!IsValidMessageID(cp->av[1], false, laxmid)) {
 	snprintf(cp->Sendid.data, cp->Sendid.size, "%d %s",
                  NNTP_FAIL_CHECK_REFUSE, cp->av[1]);
 	NCwritereply(cp, cp->Sendid.data);
@@ -1922,7 +1932,7 @@ NCtakethis(CHANNEL *cp)
     idlen = strlen(mid);
     msglen = idlen + 5; /* 3 digits + space + id + null. */
 
-    if (!IsValidMessageID(mid, false)) {
+    if (!IsValidMessageID(mid, false, laxmid)) {
         syslog(L_NOTICE, "%s bad_messageid %s", CHANname(cp),
                MaxLength(mid, mid));
         returncode = NNTP_FAIL_TAKETHIS_REJECT;
