@@ -1,4 +1,4 @@
-/*  $Id: perl.c 9930 2015-08-10 11:51:03Z iulius $
+/*  $Id: perl.c 10229 2018-01-28 21:21:21Z iulius $
 **
 **  Perl filtering support for innd.
 **
@@ -39,7 +39,9 @@
 #include "innd.h"
 
 #include <EXTERN.h>
+#pragma GCC diagnostic ignored "-Wredundant-decls"
 #include <perl.h>
+#pragma GCC diagnostic warning "-Wredundant-decls"
 #include <XSUB.h>
 #include "ppport.h"
 
