@@ -1,4 +1,4 @@
-/*  $Id: getfqdn.c,v 1.10 2003/01/19 19:58:22 rra Exp $
+/*  $Id: getfqdn.c 10235 2018-02-03 20:52:30Z eagle $
 **
 */
 
@@ -6,12 +6,13 @@
 #include "clibrary.h"
 #include <netdb.h>
 
-#include "libinn.h"
-#include "paths.h"
+#include "inn/libinn.h"
+#include "inn/paths.h"
 
 
 /*
-**  Get the fully-qualified domain name for this host.
+**  Get the fully qualified domain name for this host, as reported
+**  by the system.
 */
 char *GetFQDN(char *domain)
 {
@@ -34,9 +35,13 @@ char *GetFQDN(char *domain)
     if (strchr(buff, '.') != NULL)
 	return buff;
 
-    /* See if DNS (or /etc/hosts) gives us a full domain name. */
+    /*
+    ** See if DNS (or /etc/hosts) gives us a full domain name.  If the host
+    ** doesn't exist in DNS at all but we were given a domain name, use the
+    ** fallback of appending that domain to the hostname.
+    */
     if ((hp = gethostbyname(buff)) == NULL)
-	return NULL;
+	goto fallback;
 #if	0
     /* This code is a "feature" that allows multiple domains (NIS or
      * DNS, I'm not sure) to work with a single INN server.  However,
@@ -49,10 +54,13 @@ char *GetFQDN(char *domain)
         strlcat(temp, ".", sizeof(temp));
 	hp = gethostbyname(temp);
     }
+    if (hp == NULL) {
+        return NULL;
+    }
 #endif	/* 0 */
 
     /* First, see if the main name is a FQDN.  It should be. */
-    if (hp != NULL && strchr(hp->h_name, '.') != NULL) {
+    if (strchr(hp->h_name, '.') != NULL) {
 	if (strlen(hp->h_name) < sizeof buff - 1) {
 	    strlcpy(buff, hp->h_name, sizeof(buff));
             return buff;
@@ -76,6 +84,7 @@ char *GetFQDN(char *domain)
 	    }
 
     /* Give up:  Get the domain config param and append it. */
+fallback:
     if ((p = domain) == NULL || *p == '\0')
 	return NULL;
     if (strlen(buff) + 1 + strlen(p) > sizeof buff - 1)
