@@ -1,11 +1,11 @@
-/*  $Id: nnrpd.h 8266 2008-12-27 13:39:23Z iulius $
+/*  $Id: nnrpd.h 10229 2018-01-28 21:21:21Z iulius $
 **
 **  NetNews Reading Protocol server.
 */
 
 #include "config.h"
+#include "portable/macros.h"
 #include "portable/socket.h"
-#include "portable/time.h"
 
 #include <ctype.h>
 #include <errno.h>
@@ -13,19 +13,34 @@
 #include <syslog.h>
 #include <sys/stat.h>
 
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+#include <time.h>
+
 #include "inn/qio.h"
 #include "inn/libinn.h"
-#include "nntp.h"
+#include "inn/nntp.h"
 #include "inn/paths.h"
 #include "inn/storage.h"
 #include "inn/vector.h"
 #include "inn/timer.h"
 
+#ifndef HAVE_SASL_SASL_H
+# undef HAVE_SASL
+#endif
 #ifdef HAVE_SASL
 #include <sasl/sasl.h>
 #include <sasl/saslutil.h>
 #endif
 
+#if !defined(HAVE_ZLIB_H)
+# undef HAVE_ZLIB
+#endif
+#if defined(HAVE_ZLIB)
+# include <zlib.h>
+#endif
+
 /*
 **  A range of article numbers.
 */
@@ -62,20 +77,21 @@ typedef struct _ACCESSGROUP {
     int spoolfirst;
     int checkincludedtext;
     int clienttimeout;
-    long localmaxartsize;
+    unsigned long localmaxartsize;
     int readertrack;
     int strippostcc;
-    int addnntppostinghost;
-    int addnntppostingdate;
+    int addinjectiondate;
+    int addinjectionpostingaccount;
+    int addinjectionpostinghost;
     char *nnrpdposthost;
-    int nnrpdpostport;
+    unsigned long nnrpdpostport;
     int nnrpdoverstats;
     int backoff_auth;
     char *backoff_db;
-    long backoff_k;
-    long backoff_postfast;
-    long backoff_postslow;
-    long backoff_trigger;
+    unsigned long backoff_k;
+    unsigned long backoff_postfast;
+    unsigned long backoff_postslow;
+    unsigned long backoff_trigger;
     int nnrpdcheckart;
     int nnrpdauthsender;
     int virtualhost;
@@ -151,7 +167,7 @@ enum timer {
 
 EXTERN bool	PERMauthorized;
 EXTERN bool     PERMcanauthenticate;
-#ifdef HAVE_SSL
+#if defined(HAVE_OPENSSL) || defined(HAVE_SASL)
 EXTERN bool     PERMcanauthenticatewithoutSSL;
 #endif
 EXTERN bool	PERMcanpost;
@@ -208,34 +224,37 @@ EXTERN struct vector *OVextra;
 EXTERN int	overhdr_xref;
 EXTERN bool     LLOGenable;
 
-extern const char	*ARTpost(char *article, char *idbuff, bool ihave,
-				 bool *permanent);
+extern const char	*ARTpost(char *article, char *idbuff, bool *permanent);
 extern void		ARTclose(void);
 extern int		TrimSpaces(char *line);
-extern char		*Glom(char **av);
-extern int		Argify(char *line, char ***argvp);
 extern void		InitBackoffConstants(void);
 extern char		*PostRecFilename(char *ip, char *user);
 extern int		LockPostRec(char *path);
-extern int		LockPostRec(char *path);
 extern void		UnlockPostRec(char *path);
 extern int		RateLimit(long *sleeptime, char *path);
-extern void		ExitWithStats(int x, bool readconf);
-extern char		*GetHeader(const char *header);
+extern void		ExitWithStats(int x, bool readconf)
+    __attribute__ ((__noreturn__));
+extern char		*GetHeader(const char *header, bool stripspaces);
 extern void		GRPreport(void);
 extern bool		NGgetlist(char ***argvp, char *list);
 extern bool		PERMartok(void);
-extern void		PERMgetaccess(char *nnrpaccess);
+extern void             PERMgetinitialaccess(char *readersconf);
+extern void             PERMgetaccess(bool initialconnection);
 extern void		PERMgetpermissions(void);
 extern void		PERMlogin(char *uname, char *pass, int* code, char *errorstr);
 extern bool		PERMmatch(char **Pats, char **list);
 extern bool		ParseDistlist(char ***argvp, char *list);
 extern void 		SetDefaultAccess(ACCESSGROUP*);
-extern void		Reply(const char *fmt, ...);
-extern void             Printf(const char *fmt, ...);
+extern void		Reply(const char *fmt, ...)
+    __attribute__((__format__(printf, 1, 2)));
+extern void             Printf(const char *fmt, ...)
+    __attribute__((__format__(printf, 1, 2)));
 
 extern void             CMDauthinfo     (int ac, char** av);
 extern void             CMDcapabilities (int ac, char** av);
+#if defined(HAVE_ZLIB)
+extern void             CMDcompress     (int ac, char** av);
+#endif
 extern void             CMDdate         (int ac, char** av);
 extern void             CMDfetch        (int ac, char** av);
 extern void             CMDgroup        (int ac, char** av);
@@ -247,11 +266,12 @@ extern void             CMDnewnews      (int ac, char** av);
 extern void             CMDnextlast     (int ac, char** av);
 extern void             CMDover         (int ac, char** av);
 extern void             CMDpost         (int ac, char** av);
-extern void             CMDquit         (int ac, char** av);
+extern void             CMDquit         (int ac, char** av)
+    __attribute__ ((__noreturn__));
 extern void             CMDxgtitle      (int ac, char** av);
 extern void             CMDpat          (int ac, char** av);
 extern void             CMD_unimp       (int ac, char** av);
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
 extern void             CMDstarttls     (int ac, char** av);
 #endif
 
@@ -292,6 +312,7 @@ void PY_dynamic_init (char* file);
 
 void line_free(struct line *);
 void line_init(struct line *);
+void line_reset(struct line *);
 READTYPE line_read(struct line *, int, const char **, size_t *, size_t *);
 
 #ifdef HAVE_SASL
@@ -302,3 +323,25 @@ extern sasl_callback_t sasl_callbacks[];
 void SASLauth(int ac, char *av[]);
 void SASLnewserver(void);
 #endif /* HAVE_SASL */
+
+#if defined(HAVE_SASL) || defined(HAVE_ZLIB)
+bool IsValidAlgorithm(const char *);
+#endif /* HAVE_SASL || HAVE_ZLIB */
+
+#if defined(HAVE_ZLIB)
+extern bool compression_layer_on;
+extern bool tls_compression_on;
+/* (De)compress streams and related variables. */
+extern z_stream *zstream_in;
+extern z_stream *zstream_out;
+extern bool zstream_inflate_needed;
+extern bool zstream_flush_needed;
+/* (De)compress buffers and related variables. */
+extern unsigned char *zbuf_in;
+extern size_t zbuf_in_allocated;
+extern size_t zbuf_in_size;
+extern unsigned char *zbuf_out;
+extern size_t zbuf_out_size;
+
+bool zlib_init(void);
+#endif /* HAVE_ZLIB */
