@@ -1,4 +1,4 @@
-/*  $Id: perl.c 8442 2009-05-04 19:11:52Z iulius $
+/*  $Id: perl.c 10229 2018-01-28 21:21:21Z iulius $
 **
 **  Embedded Perl support for INN.
 **
@@ -18,23 +18,23 @@
 
 #include "inn/innconf.h"
 #include "nnrpd.h"
+#include "inn/nntp.h"
 #include "inn/paths.h"
 #include "post.h"
 
-#include "nntp.h"
-
 /* Skip this entire file if DO_PERL (./configure --with-perl) isn't set. */
 #ifdef DO_PERL
 
 #include <EXTERN.h>
+#pragma GCC diagnostic ignored "-Wredundant-decls"
 #include <perl.h>
+#pragma GCC diagnostic warning "-Wredundant-decls"
 #include <XSUB.h>
 #include "ppport.h"
 
 #include "innperl.h"
 
 extern HEADER Table[], *EndOfTable;
-extern char PERMuser[];
 
 extern char **OtherHeaders;
 extern int OtherCount;
@@ -53,6 +53,7 @@ HandleHeaders(char *article)
 {
     dSP;
     HEADER *hp;
+    HV *attribs;
     HV *hdr;
     SV *body;
     int rc;
@@ -65,6 +66,8 @@ HandleHeaders(char *article)
     SV *modswitch;
     int OtherSize;
     char *argv[] = { NULL };
+    bool failure;
+    SV *errsv;
 
     if(!PerlLoaded) {
         loadPerl();
@@ -78,18 +81,27 @@ HandleHeaders(char *article)
         syslog(L_ERROR,"Whoops.  Can't open error log: %m");
     }
 #endif /* DEBUG_MODIFY */
-   
+
     ENTER;
     SAVETMPS;
-   
-    /* Create the Perl hash. */
+
+    /* Create the Perl attributes hash. */
+    attribs = perl_get_hv("attributes", true);
+    (void) hv_store(attribs, "hostname", 8, newSVpv(Client.host, 0), 0);
+    (void) hv_store(attribs, "ipaddress", 9, newSVpv(Client.ip, 0), 0);
+    (void) hv_store(attribs, "port", 4, newSViv(Client.port), 0);
+    (void) hv_store(attribs, "interface", 9, newSVpv(Client.serverhost, 0), 0);
+    (void) hv_store(attribs, "intipaddr", 9, newSVpv(Client.serverip, 0), 0);
+    (void) hv_store(attribs, "intport", 7, newSViv(Client.serverport), 0);
+
+    /* Create the Perl header hash. */
     hdr = perl_get_hv("hdr", true);
     for (hp = Table; hp < EndOfTable; hp++) {
         if (hp->Body)
             (void) hv_store(hdr, (char *) hp->Name, strlen(hp->Name),
                      newSVpv(hp->Body, 0), 0);
     }
-   
+
     /* Also store other headers. */
     OtherSize = OtherCount;
     for (i = 0; i < OtherCount; i++) {
@@ -110,12 +122,13 @@ HandleHeaders(char *article)
 
     /* Store user. */
     sv_setpv(perl_get_sv("user", true), PERMuser);
-   
+
     /* Store body. */
     body = perl_get_sv("body", true);
     sv_setpv(body, article);
 
     /* Call the filtering function. */
+    /* No need for PUSHMARK(SP) with call_argv(). */
     rc = perl_call_argv("filter_post", G_EVAL|G_SCALAR, argv);
 
     SPAGAIN;
@@ -127,7 +140,7 @@ HandleHeaders(char *article)
         HeadersModified = true;
         i = 0;
 
-#ifdef DEBUG_MODIFY     
+#ifdef DEBUG_MODIFY
         dumpTable("Before mod");
 #endif /* DEBUG_MODIFY */
 
@@ -137,13 +150,13 @@ HandleHeaders(char *article)
              * new values. */
             p = HePV(scan, len);
             s = SvPV(HeVAL(scan), PL_na);
-#ifdef DEBUG_MODIFY     
+#ifdef DEBUG_MODIFY
             fprintf(flog,"Hash iter: '%s','%s'\n", p, s);
 #endif /* DEBUG_MODIFY */
 
             /* See if it is a table header. */
             for (hp = Table; hp < EndOfTable; hp++) {
-                if (strncasecmp(p, hp->Name, hp->Size) == 0) {
+                if (strcasecmp(p, hp->Name) == 0) {
                     char *copy = xstrdup(s);
                     HDR_SET(hp - Table, copy);
                     hp->Len = TrimSpaces(hp->Value);
@@ -176,27 +189,42 @@ HandleHeaders(char *article)
 #endif /* DEBUG_MODIFY */
     }
 
+    hv_undef(attribs);
     hv_undef(hdr);
     sv_setsv(body, &PL_sv_undef);
 
     buf[0] = '\0';
-   
-    if (SvTRUE(ERRSV)) {    /* Check $@. */
+
+    /* Check $@. */
+    errsv = ERRSV;
+    if (SvTRUE(errsv)) {
+        failure = true;
         syslog(L_ERROR, "Perl function filter_post died: %s",
-               SvPV(ERRSV, PL_na));
+               SvPV(errsv, PL_na));
         (void)POPs;
-        PerlFilter(false);
-    } else if (rc == 1) {
-        p = POPp;
-        if (p != NULL && *p != '\0')
-            strlcpy(buf, p, sizeof(buf));
+    } else {
+        failure = false;
+        if (rc == 1) {
+            p = POPp;
+            if (p != NULL && *p != '\0')
+                strlcpy(buf, p, sizeof(buf));
+        }
     }
 
+    PUTBACK;
     FREETMPS;
     LEAVE;
-   
-    if (buf[0] != '\0') 
+
+    if (failure)
+        PerlFilter(false);
+
+    if (buf[0] != '\0')
         return buf;
+
+#ifdef DEBUG_MODIFY
+    fclose(flog);
+#endif /* DEBUG_MODIFY */
+
     return NULL;
 }
 
@@ -222,6 +250,7 @@ perlAccess(char *user, struct vector *access_vec)
     SV *sv;
     int rc, i;
     char *key, *val, *buffer;
+    SV *errsv;
 
     if (!PerlFilterActive)
         return;
@@ -239,6 +268,7 @@ perlAccess(char *user, struct vector *access_vec)
     (void) hv_store(attribs, "username", 8, newSVpv(user, 0), 0);
 
     PUSHMARK(SP);
+    PUTBACK;
 
     if (perl_get_cv("access", 0) == NULL) {
         syslog(L_ERROR, "Perl function access not defined");
@@ -251,8 +281,9 @@ perlAccess(char *user, struct vector *access_vec)
     SPAGAIN;
 
     if (rc == 0) { /* Error occured, same as checking $@. */
+        errsv = ERRSV;
         syslog(L_ERROR, "Perl function access died: %s",
-               SvPV(ERRSV, PL_na));
+               SvPV(errsv, PL_na));
         Reply("%d Internal error (1).  Goodbye!\r\n", NNTP_FAIL_TERMINATING);
         ExitWithStats(1, true);
     }
@@ -277,11 +308,12 @@ perlAccess(char *user, struct vector *access_vec)
         strlcat(buffer, ": \"", BIG_BUFFER);
         strlcat(buffer, val, BIG_BUFFER);
         strlcat(buffer, "\"\n", BIG_BUFFER);
- 
+
         vector_add(access_vec, buffer);
     }
 
     free(buffer);
+    hv_undef(attribs);
 
     PUTBACK;
     FREETMPS;
@@ -294,28 +326,28 @@ perlAuthInit(void)
 {
     dSP;
     int rc;
-    
+    SV *errsv;
+
     if (!PerlFilterActive)
         return;
 
+    if (perl_get_cv("auth_init", 0) == NULL) {
+        return;
+    }
+
     ENTER;
     SAVETMPS;
     PUSHMARK(SP);
-    
-    if (perl_get_cv("auth_init", 0) == NULL) {
-        syslog(L_ERROR, "Perl function auth_init not defined");
-        Reply("%d Internal error (3).  Goodbye!\r\n", NNTP_FAIL_TERMINATING);
-        ExitWithStats(1, true);
-    }
+    PUTBACK;
 
     rc = perl_call_pv("auth_init", G_EVAL|G_DISCARD);
 
     SPAGAIN;
 
-
-    if (SvTRUE(ERRSV)) {    /* Check $@. */
+    errsv = ERRSV;
+    if (SvTRUE(errsv)) {    /* Check $@. */
         syslog(L_ERROR, "Perl function authenticate died: %s",
-               SvPV(ERRSV, PL_na));
+               SvPV(errsv, PL_na));
         Reply("%d Internal error (1).  Goodbye!\r\n", NNTP_FAIL_TERMINATING);
         ExitWithStats(1, true);
     }
@@ -337,7 +369,8 @@ perlAuthenticate(char *user, char *passwd, int *code, char *errorstring, char *n
     HV *attribs;
     int rc;
     char *p;
-    
+    SV *errsv;
+
     if (!PerlFilterActive)
         *code = NNTP_FAIL_AUTHINFO_BAD;
 
@@ -358,15 +391,17 @@ perlAuthenticate(char *user, char *passwd, int *code, char *errorstring, char *n
     (void) hv_store(attribs, "intport", 7, newSViv(Client.serverport), 0);
     (void) hv_store(attribs, "username", 8, newSVpv(user, 0), 0);
     (void) hv_store(attribs, "password", 8, newSVpv(passwd, 0), 0);
-    
+
     PUSHMARK(SP);
+    PUTBACK;
     rc = perl_call_pv("authenticate", G_EVAL|G_ARRAY);
 
     SPAGAIN;
 
     if (rc == 0 ) { /* Error occurred, same as checking $@. */
+        errsv = ERRSV;
         syslog(L_ERROR, "Perl function authenticate died: %s",
-               SvPV(ERRSV, PL_na));
+               SvPV(errsv, PL_na));
         Reply("%d Internal error (1).  Goodbye!\r\n", NNTP_FAIL_TERMINATING);
         ExitWithStats(1, false);
     }
@@ -383,7 +418,7 @@ perlAuthenticate(char *user, char *passwd, int *code, char *errorstring, char *n
     if (rc == 3) {
         p = POPp;
         strlcpy(newUser, p, BIG_BUFFER);
-    } 
+    }
 
     p = POPp;
     strlcpy(errorstring, p, BIG_BUFFER);
@@ -406,7 +441,7 @@ dumpTable (char *msg)
     int i;
 
     fprintf(flog, "===BEGIN TABLE DUMP: %s\n", msg);
-      
+
     for (hp = Table; hp < EndOfTable; hp++) {
         fprintf(flog, " Name: '%s'",hp->Name);
         fflush(flog);
