@@ -1,4 +1,4 @@
-/*  $Id: tls.c 9785 2015-01-10 14:25:18Z iulius $
+/*  $Id: tls.c 10198 2017-12-10 15:22:43Z iulius $
 **
 **  tls.c -- TLSv1 functions.
 **  Copyright (C) 2000 Kenichi Okada <okada@opaopa.org>.
@@ -23,14 +23,12 @@
 #include "nnrpd.h"
 #include "inn/innconf.h"
 
-/* Outside the ifdef so that make depend works even ifndef HAVE_SSL. */
+/* Outside the ifdef so that make depend works even ifndef HAVE_OPENSSL. */
 #include "tls.h"
 
-#ifdef HAVE_SSL
+#ifdef HAVE_OPENSSL
 
 /* We must keep some of the info available. */
-static const char hexcodes[] = "0123456789ABCDEF";
-
 static bool tls_initialized = false;
 
 static int verify_depth;
@@ -83,20 +81,20 @@ apps_ssl_info_callback(const SSL *s, int where, int ret)
 
     if (where & SSL_CB_LOOP) {
 	if (tls_serverengine && (tls_loglevel >= 2))
-	    Printf("%s:%s", str, SSL_state_string_long(s));
+            syslog(L_NOTICE, "%s:%s", str, SSL_state_string_long(s));
     } else if (where & SSL_CB_ALERT) {
 	str = (where & SSL_CB_READ) ? "read" : "write";
 	if ((tls_serverengine && (tls_loglevel >= 2)) ||
 	    ((ret & 0xff) != SSL3_AD_CLOSE_NOTIFY))
-	  Printf("SSL3 alert %s:%s:%s", str,
+            syslog(L_NOTICE, "SSL3 alert %s:%s:%s", str,
 		 SSL_alert_type_string_long(ret),
 		 SSL_alert_desc_string_long(ret));
     } else if (where & SSL_CB_EXIT) {
 	if (ret == 0)
-	    Printf("%s:failed in %s",
+            syslog(L_ERROR, "%s:failed in %s",
 		     str, SSL_state_string_long(s));
 	else if (ret < 0) {
-	    Printf("%s:error in %s",
+            syslog(L_ERROR, "%s:error in %s",
 		     str, SSL_state_string_long(s));
 	}
     }
@@ -218,7 +216,10 @@ tmp_dh_cb(SSL *s UNUSED, int export UNUSED, int keylength)
 	default:
 		/* We should check current keylength vs. requested keylength
 		 * also, this is an extremely expensive operation! */
-		dh = DH_generate_parameters(keylength, DH_GENERATOR_2, NULL, NULL);
+                dh = DH_new();
+                if (dh != NULL) {
+                    DH_generate_parameters_ex(dh, keylength, DH_GENERATOR_2, NULL);
+                }
 		r = dh;
 	}
 
@@ -243,9 +244,15 @@ verify_callback(int ok, X509_STORE_CTX * ctx)
     err = X509_STORE_CTX_get_error(ctx);
     depth = X509_STORE_CTX_get_error_depth(ctx);
 
-    X509_NAME_oneline(X509_get_subject_name(err_cert), buf, sizeof(buf));
-    if ((tls_serveractive) && (tls_loglevel >= 1))
-      Printf("Peer cert verify depth=%d %s", depth, buf);
+    if ((tls_serveractive) && (tls_loglevel >= 1)) {
+        if (err_cert != NULL) {
+            X509_NAME_oneline(X509_get_subject_name(err_cert), buf, sizeof(buf));
+            syslog(L_NOTICE, "Peer cert verify depth=%d %s", depth, buf);
+        } else {
+            syslog(L_NOTICE, "Peer cert verify depth=%d <no cert>", depth);
+        }
+    }
+    
     if (ok==0)
     {
       syslog(L_NOTICE, "verify error:num=%d:%s", err,
@@ -259,9 +266,10 @@ verify_callback(int ok, X509_STORE_CTX * ctx)
 	    verify_error = X509_V_ERR_CERT_CHAIN_TOO_LONG;
 	}
     }
-    switch (ctx->error) {
+
+    switch (err) {
     case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT:
-	X509_NAME_oneline(X509_get_issuer_name(ctx->current_cert), buf, sizeof(buf));
+	X509_NAME_oneline(X509_get_issuer_name(err_cert), buf, sizeof(buf));
 	syslog(L_NOTICE, "issuer= %s", buf);
 	break;
     case X509_V_ERR_CERT_NOT_YET_VALID:
@@ -274,7 +282,7 @@ verify_callback(int ok, X509_STORE_CTX * ctx)
 	break;
     }
     if ((tls_serveractive) && (tls_loglevel >= 1))
-      Printf("verify return:%d", ok);
+        syslog(L_NOTICE, "verify return:%d", ok);
 
     return (ok);
 }
@@ -342,14 +350,14 @@ tls_dump(const char *s, int len)
 	/* If this is the last call, then update the ddt_dump thing so that
          * we will move the selection point in the debug window. */
 	if (tls_loglevel>0)
-	  Printf("%s", buf);
+            syslog(L_NOTICE, "%s", buf);
 	ret += strlen(buf);
     }
 #ifdef TRUNCATE
     if (trunc > 0) {
 	snprintf(buf, sizeof(buf), "%04x - <SPACES/NULS>\n", len+ trunc);
 	if (tls_loglevel>0)
-	  Printf("%s", buf);
+            syslog(L_NOTICE, "%s", buf);
 	ret += strlen(buf);
     }
 #endif
@@ -385,15 +393,13 @@ set_cert_stuff(SSL_CTX * ctx, char *cert_file, char *key_file)
 	    return (0);
 	}
 
-        /* Check that the key file is a real file, not readable by
-         * everyone.  If the mode is 440 or 640, make sure the group owner
-         * is the news group (to prevent the failure case of having news:users
-         * as the owner and group. */
-	if (!S_ISREG(buf.st_mode) || (buf.st_mode & 0137) != 0
-            || ((buf.st_mode & 0040) != 0 && buf.st_gid != getegid())) {
+        /* Check that the key file is a real file, isn't world-readable, and
+         * that we can read it. */
+	if (!S_ISREG(buf.st_mode) || (buf.st_mode & 0007) != 0
+            || access(key_file, R_OK) < 0) {
 	    syslog(L_ERROR, "bad ownership or permissions on private key"
-                   " '%s':  private key must be mode 640 at most, and readable"
-                   " by the news group only", key_file);
+                   " '%s': private key must be a regular file, readable by"
+                   " nnrpd, and not world-readable", key_file);
 	    return (0);
 	}
 
@@ -492,12 +498,19 @@ tls_init_serverengine(int verifydepth, int askcert, int requirecert,
       return (0);				/* Already running. */
 
     if (tls_loglevel >= 2)
-      Printf("starting TLS engine");
+      syslog(L_NOTICE, "starting TLS engine");
 
+/* New functions have been introduced in OpenSSL 1.1.0. */
+#if OPENSSL_VERSION_NUMBER < 0x010100000L
     SSL_load_error_strings();
     SSLeay_add_ssl_algorithms();
-
     CTX = SSL_CTX_new(SSLv23_server_method());
+#else
+    OPENSSL_init_ssl(OPENSSL_INIT_LOAD_SSL_STRINGS
+                     | OPENSSL_INIT_LOAD_CRYPTO_STRINGS, NULL);
+    CTX = SSL_CTX_new(TLS_server_method());
+#endif
+
     if (CTX == NULL) {
       return (-1);
     };
@@ -519,7 +532,7 @@ tls_init_serverengine(int verifydepth, int askcert, int requirecert,
     if ((!SSL_CTX_load_verify_locations(CTX, CAfile, CApath)) ||
 	(!SSL_CTX_set_default_verify_paths(CTX))) {
       if (tls_loglevel >= 2)
-	Printf("TLS engine: cannot load CA data\n");
+          syslog(L_ERROR, "TLS engine: cannot load CA data");
       return (-1);
     }
     
@@ -534,7 +547,7 @@ tls_init_serverengine(int verifydepth, int askcert, int requirecert,
     
     if (!set_cert_stuff(CTX, s_cert_file, s_key_file)) {
       if (tls_loglevel >= 2)
-	Printf("TLS engine: cannot load cert/key data\n");
+          syslog(L_ERROR, "TLS engine: cannot load cert/key data");
       return (-1);
     }
 
@@ -565,11 +578,16 @@ tls_init_serverengine(int verifydepth, int askcert, int requirecert,
      }
 #endif /* HAVE_OPENSSL_ECC */
 
-    if (prefer_server_ciphers) {
 #ifdef SSL_OP_CIPHER_SERVER_PREFERENCE
+    if (prefer_server_ciphers) {
         SSL_CTX_set_options(CTX, SSL_OP_CIPHER_SERVER_PREFERENCE);
+    } else {
+#if OPENSSL_VERSION_NUMBER >= 0x0009080dfL
+        /* Function first added in OpenSSL 0.9.8m. */
+        SSL_CTX_clear_options(CTX, SSL_OP_CIPHER_SERVER_PREFERENCE);
 #endif
     }
+#endif
 
     if ((tls_proto_vect != NULL) && (tls_proto_vect->count > 0)) {
         for (i = 0; i < tls_proto_vect->count; i++) {
@@ -584,6 +602,8 @@ tls_init_serverengine(int verifydepth, int askcert, int requirecert,
                     tls_protos |= INN_TLS_TLSv1_1;
                 } else if (strcmp(tls_proto_vect->strings[i], "TLSv1.2") == 0) {
                     tls_protos |= INN_TLS_TLSv1_2;
+                } else if (strcmp(tls_proto_vect->strings[i], "TLSv1.3") == 0) {
+                    tls_protos |= INN_TLS_TLSv1_3;
                 } else {
                     syslog(L_ERROR, "TLS engine: unknown protocol '%s' in tlsprotocols",
                            tls_proto_vect->strings[i]);
@@ -591,9 +611,9 @@ tls_init_serverengine(int verifydepth, int askcert, int requirecert,
             }
         }
     } else {
-        /* Default value:  allow all protocols. */
-        tls_protos = (INN_TLS_SSLv2 | INN_TLS_SSLv3 | INN_TLS_TLSv1
-                      | INN_TLS_TLSv1_1 | INN_TLS_TLSv1_2);
+        /* Default value:  allow only TLS protocols. */
+        tls_protos = (INN_TLS_TLSv1 | INN_TLS_TLSv1_1 | INN_TLS_TLSv1_2
+                      | INN_TLS_TLSv1_3);
     }
 
     if ((tls_protos & INN_TLS_SSLv2) == 0) {
@@ -620,6 +640,12 @@ tls_init_serverengine(int verifydepth, int askcert, int requirecert,
 #endif
     }
 
+    if ((tls_protos & INN_TLS_TLSv1_3) == 0) {
+#ifdef SSL_OP_NO_TLSv1_3
+        SSL_CTX_set_options(CTX, SSL_OP_NO_TLSv1_3);
+#endif
+    }
+
     if (tls_ciphers != NULL) {
         if (SSL_CTX_set_cipher_list(CTX, tls_ciphers) == 0) {
             syslog(L_ERROR, "TLS engine: cannot set cipher list");
@@ -627,9 +653,18 @@ tls_init_serverengine(int verifydepth, int askcert, int requirecert,
         }
     }
 
-    if (!tls_compression) {
+    if (tls_compression) {
+#if defined(SSL_OP_NO_COMPRESSION) && OPENSSL_VERSION_NUMBER >= 0x0009080dfL
+        /* Function first added in OpenSSL 0.9.8m. */
+        SSL_CTX_clear_options(CTX, SSL_OP_NO_COMPRESSION);
+#endif
+    } else {
 #ifdef SSL_OP_NO_COMPRESSION
+        /* Option implemented in OpenSSL 1.0.0. */
         SSL_CTX_set_options(CTX, SSL_OP_NO_COMPRESSION);
+#elif OPENSSL_VERSION_NUMBER >= 0x00090800fL
+        /* Workaround for OpenSSL 0.9.8. */
+        sk_SSL_COMP_zero(SSL_COMP_get_compression_methods());
 #endif
     }
 
@@ -704,13 +739,15 @@ bio_dump_cb(BIO * bio, int cmd, const char *argp, int argi, long argl UNUSED,
 	return (ret);
 
     if (cmd == (BIO_CB_READ | BIO_CB_RETURN)) {
-	Printf("read from %08lX [%08lX] (%d bytes => %ld (0x%X))", (long unsigned int) bio, (long unsigned int) argp,
-		 argi, ret, (unsigned int) ret);
+        syslog(L_NOTICE, "read from %08lX [%08lX] (%d bytes => %ld (0x%lX))",
+               (unsigned long) bio, (unsigned long) argp,
+               argi, ret, (unsigned long) ret);
 	tls_dump(argp, (int) ret);
 	return (ret);
     } else if (cmd == (BIO_CB_WRITE | BIO_CB_RETURN)) {
-	Printf("write to %08lX [%08lX] (%d bytes => %ld (0x%X))", (long unsigned int) bio, (long unsigned int) argp,
-		 argi, ret, (unsigned int) ret);
+        syslog(L_NOTICE, "write to %08lX [%08lX] (%d bytes => %ld (0x%lX))",
+               (unsigned long) bio, (unsigned long) argp,
+               argi, ret, (unsigned long) ret);
 	tls_dump(argp, (int) ret);
     }
     return (ret);
@@ -741,7 +778,7 @@ tls_start_servertls(int readfd, int writefd)
       return (-1);
     }
     if (tls_loglevel >= 1)
-	Printf("setting up TLS connection");
+	syslog(L_NOTICE, "setting up TLS connection");
 
     if (tls_conn == NULL)
     {
@@ -853,4 +890,4 @@ SSL_writev (SSL *ssl, const struct iovec *vector, int count)
   return SSL_write (ssl, buffer, bytes);
 }
 
-#endif /* HAVE_SSL */
+#endif /* HAVE_OPENSSL */
