@@ -1,7 +1,7 @@
 ##########################################################################
 # INN module for innreport (3.*).
 #
-# $Id: innreport_inn.pm 9636 2014-05-14 18:06:52Z iulius $
+# $Id: innreport_inn.pm 10259 2018-03-08 16:19:17Z iulius $
 #
 # Sample file tested with INN 2.5, 2.4, 2.3, 2.2, 1.7.2 and 1.5.1.
 #
@@ -28,7 +28,7 @@ my %ctlinnd = ('a', 'addhist',     'D', 'allow',
 	       'i', 'hangup',      's', 'mode',
 	       'j', 'name',        'k', 'newgroup',
 	       'l', 'param',       'm', 'pause',
-	       'v', 'readers',     't', 'refile',
+	       'v', 'readers',     'H', 'stathist',
 	       'C', 'reject',      'o', 'reload',
 	       'n', 'renumber',    'z', 'reserve',
 	       'p', 'rmgroup',     'A', 'send',
@@ -36,7 +36,9 @@ my %ctlinnd = ('a', 'addhist',     'D', 'allow',
 	       'r', 'throttle',    'w', 'trace',
 	       'x', 'xabort',      'y', 'xexec',
 	       'E', 'logmode',     'F', 'feedinfo',
-	       'T', 'filter',      'P', 'perl',);
+	       'S', 'status',      'P', 'perl',
+               'L', 'lowmark',     'Y', 'python',
+               'Z', 'timer');
 
 my %timer_names = (idle     => 'idle',
                    hishave  => 'history lookup',
@@ -1068,7 +1070,7 @@ sub collect($$$$$$) {
     return 1 if $left =~ m/ hostChkCxn - now/o;
     # loading path_to_config_file/innfeed.conf
     return 1 if $left =~ m/loading /o;
-    # Finnaly, to avoid problems with strange error lines, ignore them.
+    # Finally, to avoid problems with strange error lines, ignore them.
     #return 1 if ($left =~ /ME /);
   }
   ########
@@ -1507,6 +1509,7 @@ sub collect($$$$$$) {
     # other logs that should not be reported as errors
     return 1 if $left =~ /^\S+ auth also-log: /o;
     return 1 if $left =~ /^\S+ res also-log: /o;
+    return 1 if $left =~ /^\S+ rejected by rule /o;
     # connect
     if ($left =~ /(\S+) (\([0-9a-fA-F:.]*\) )?connect(?: - port \d+)?$/o) {
       my $cust = $1;
@@ -1938,6 +1941,7 @@ sub collect($$$$$$) {
     if ($left =~ /Article <[^>]*>: (.*) \(ID [[:xdigit:]]*\) not in keyring/o) {
       $nocem_badsigs{$1}++;
       $nocem_goodsigs{$1} = 0 unless ($nocem_goodsigs{$1});
+      $nocem_totalids{$1} = 0 unless ($nocem_totalids{$1});
       $nocem_totalbad++;
       $nocem_lastid = $1;
       return 1;
@@ -1945,6 +1949,7 @@ sub collect($$$$$$) {
     if ($left =~ /Article <[^>]*>: bad signature from (.*)/o) {
       $nocem_badsigs{$1}++;
       $nocem_goodsigs{$1} = 0 unless ($nocem_goodsigs{$1});
+      $nocem_totalids{$1} = 0 unless ($nocem_totalids{$1});
       $nocem_totalbad++;
       $nocem_lastid = $1;
       return 1;
@@ -1952,6 +1957,7 @@ sub collect($$$$$$) {
     if ($left =~ /Article <[^>]*>: malformed signature/o) {
       $nocem_badsigs{'N/A'}++;
       $nocem_goodsigs{'N/A'} = 0 unless ($nocem_goodsigs{'N/A'});
+      $nocem_totalids{'N/A'} = 0 unless ($nocem_totalids{'N/A'});
       $nocem_totalbad++;
       $nocem_lastid = 'N/A';
       return 1;
