@@ -1,4 +1,4 @@
-/*  $Id: cnfs.c 9874 2015-05-23 11:36:25Z iulius $
+/*  $Id: cnfs.c 10152 2017-06-05 12:27:29Z iulius $
 **
 **  Storage manager module for Cyclic News File System method.
 */
@@ -6,7 +6,6 @@
 #include "config.h"
 #include "clibrary.h"
 #include "portable/mmap.h"
-#include "portable/time.h"
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -17,6 +16,12 @@
 #include <sys/stat.h>
 #include <sys/uio.h>
 
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+#include <time.h>
+
+#include "inn/fdflag.h"
 #include "inn/innconf.h"
 #include "inn/messages.h"
 #include "inn/mmap.h"
@@ -82,7 +87,8 @@ cnfs_explaintoken(const TOKEN token)
     uint32_t            block;
     uint32_t            cycnum;
 
-    snprintf(cycbuffname, sizeof(cycbuffname), "%s", token.token);
+    snprintf(cycbuffname, sizeof(cycbuffname), "%.*s",
+             (int) sizeof(cycbuffname)-1, token.token);
 
     if ((cycbuff = CNFSgetcycbuffbyname(cycbuffname)) == NULL) {
         blksz = CNFS_DFL_BLOCKSIZE;
@@ -600,7 +606,7 @@ static bool CNFSinit_disks(CYCBUFF *cycbuff) {
             syswarn("CNFS: ERROR opening '%s' O_RDONLY", cycbuff->path);
 	    return false;
 	} else {
-	    close_on_exec(fd, true);
+	    fdflag_close_exec(fd, true);
 	    cycbuff->fd = fd;
 	}
     }
