@@ -1,4 +1,4 @@
-/* $Id: basic.h 10396 2020-11-12 20:19:41Z iulius $
+/* $Id: basic.h 10268 2018-03-15 21:40:48Z iulius $
  *
  * Basic utility routines for the TAP protocol.
  *
@@ -6,7 +6,7 @@
  * documentation is at <https://www.eyrie.org/~eagle/software/c-tap-harness/>.
  *
  * Written by Russ Allbery <eagle@eyrie.org>
- * Copyright 2009-2019 Russ Allbery <eagle@eyrie.org>
+ * Copyright 2009-2018 Russ Allbery <eagle@eyrie.org>
  * Copyright 2001-2002, 2004-2008, 2011-2012, 2014
  *     The Board of Trustees of the Leland Stanford Junior University
  *
@@ -34,9 +34,9 @@
 #ifndef TAP_BASIC_H
 #define TAP_BASIC_H 1
 
-#include <stdarg.h> /* va_list */
-#include <stddef.h> /* size_t */
 #include <tap/macros.h>
+#include <stdarg.h>             /* va_list */
+#include <stddef.h>             /* size_t */
 
 /*
  * Used for iterating through arrays.  ARRAY_SIZE returns the number of
@@ -44,8 +44,8 @@
  * ARRAY_END returns a pointer to the element past the end (ISO C99 makes it
  * legal to refer to such a pointer as long as it's never dereferenced).
  */
-#define ARRAY_SIZE(array) (sizeof(array) / sizeof((array)[0]))
-#define ARRAY_END(array)  (&(array)[ARRAY_SIZE(array)])
+#define ARRAY_SIZE(array)       (sizeof(array) / sizeof((array)[0]))
+#define ARRAY_END(array)        (&(array)[ARRAY_SIZE(array)])
 
 BEGIN_DECLS
 
@@ -103,7 +103,8 @@ int ok(int success, const char *format, ...)
     __attribute__((__format__(printf, 2, 3)));
 int okv(int success, const char *format, va_list args)
     __attribute__((__format__(printf, 2, 0)));
-void skip(const char *reason, ...) __attribute__((__format__(printf, 1, 2)));
+void skip(const char *reason, ...)
+    __attribute__((__format__(printf, 1, 2)));
 
 /*
  * Report the same status on, or skip, the next count tests.  ok_block()
@@ -113,8 +114,8 @@ int ok_block(unsigned long count, int success, const char *format, ...)
     __attribute__((__format__(printf, 3, 4)));
 void skip_block(unsigned long count, const char *reason, ...)
     __attribute__((__format__(printf, 2, 3)));
-
 #endif
+
 /*
  * Compare two values.  Returns true if the test passes and false if it fails.
  * is_bool takes an int since the bool type isn't fully portable yet, but
@@ -151,8 +152,10 @@ int sysdiag(const char *format, ...)
  * diag().  Nul characters are not supported in these files and will result in
  * truncated output.
  */
-void diag_file_add(const char *file) __attribute__((__nonnull__));
-void diag_file_remove(const char *file) __attribute__((__nonnull__));
+void diag_file_add(const char *file)
+    __attribute__((__nonnull__));
+void diag_file_remove(const char *file)
+    __attribute__((__nonnull__));
 
 /* Allocate memory, reporting a fatal error with bail on failure. */
 void *bcalloc(size_t, size_t)
@@ -168,14 +171,6 @@ char *bstrdup(const char *)
 char *bstrndup(const char *, size_t)
     __attribute__((__malloc__, __nonnull__, __warn_unused_result__));
 
-/*
- * Macros that cast the return value from b* memory functions, making them
- * usable in C++ code and providing some additional type safety.
- */
-#define bcalloc_type(n, type) ((type *) bcalloc((n), sizeof(type)))
-#define breallocarray_type(p, n, type) \
-    ((type *) breallocarray((p), (n), sizeof(type)))
-
 /*
  * Find a test file under C_TAP_BUILD or C_TAP_SOURCE, returning the full
  * path.  The returned path should be freed with test_file_path_free().
@@ -188,7 +183,8 @@ void test_file_path_free(char *path);
  * Create a temporary directory relative to C_TAP_BUILD and return the path.
  * The returned path should be freed with test_tmpdir_free().
  */
-char *test_tmpdir(void) __attribute__((__malloc__, __warn_unused_result__));
+char *test_tmpdir(void)
+    __attribute__((__malloc__, __warn_unused_result__));
 void test_tmpdir_free(char *path);
 
 /*
@@ -199,19 +195,10 @@ void test_tmpdir_free(char *path);
  * The function must return void and will be passed two arguments: an int that
  * will be true if the test completed successfully and false otherwise, and an
  * int that will be true if the cleanup function is run in the primary process
- * (the one that called plan or plan_lazy) and false otherwise.  If
- * test_cleanup_register_with_data is used instead, a generic pointer can be
- * provided and will be passed to the cleanup function as a third argument.
- *
- * test_cleanup_register_with_data is the better API and should have been the
- * only API.  test_cleanup_register was an API error preserved for backward
- * cmpatibility.
+ * (the one that called plan or plan_lazy) and false otherwise.
  */
 typedef void (*test_cleanup_func)(int, int);
-typedef void (*test_cleanup_func_with_data)(int, int, void *);
-
-void test_cleanup_register(test_cleanup_func) __attribute__((__nonnull__));
-void test_cleanup_register_with_data(test_cleanup_func_with_data, void *)
+void test_cleanup_register(test_cleanup_func)
     __attribute__((__nonnull__));
 
 END_DECLS
